function [mpcobj,projname,taskname,mpcname]=mpc_loadobj(filename,projname,taskname,mpcname)
% MPC_LOADOBJ Load MPC objects from a MAT file generated by the GUI
%
% mpcobjlist=MPC_LOADOBJ(filename) gets a list of all MPC objects stored
% in a MAT file. This can be a regular MAT file or a file generated by 
% the GUI.
%
% mpcobjlist=MPC_LOADOBJ(filename,projname) gets a list of all MPC objects stored
% in a MAT file generated by the GUI that belong to a given project 
%
% mpcobjlist=MPC_LOADOBJ(filename,projname,taskname) only lists those belonging
% to a given task
%
% mpcobj=MPC_LOADOBJ(filename,projname,taskname,mpcname) gets the actual
% MPC object corresponding to a given project, task, and controller.

%   Authors: A. Bemporad
%   Copyright 1986-2004 The MathWorks, Inc.
%   $Revision: 1.1.6.2 $  $Date: 2004/04/10 23:39:17 $

if nargin<1,
    error('mpc:mpc_loadobj:nargin','Not enough input arguments. At least a file name must be supplied');
end

mpcobj={};
if nargin<4,
    mpcname=[];
    if nargin<3,
        taskname=[];
        if nargin<2,
            projname=[];
        end
    end
end

file=filename;
if ~strcmp(lower(file(end-3:end)),'.mat'),
    file=[file '.mat'];
end
aux=which(file);
if isempty(aux),
    error('mpc:mpc_loadobj:nofile','MAT file not found');
    return
end

if ispc,
    theslash='\';
else
    theslash='/';
end

theslashes=strfind(aux,theslash);
filepath=aux(1:theslashes(end));
file=file(1:end-4); % Takes out '.mat'

% Get list of MPC objects in the given project and task
try
    load([filepath file]);
catch
    error('mpc:mpc_loadobj:load','Cannot load project file');
end
    
if ~exist('Projects') | ~isa(Projects,'mpcnodes.MPCGUI'),
    
    %warning('mpc:mpc_loadobj:notGUI','MAT File does not contain an MPC design from GUI');

    % Maybe it's a standard MAT file

    % List the names of all MPC objects stored in the MAT file
    %allvars=whos('-file',[filepath file]);
    allvars=whos;
    j=0;
    for i=1:length(allvars),
        if strcmp(allvars(i).class,'mpc'),
            j=j+1;
            mpcobj{j,1}=allvars(i).name;
        end
    end
    if nargin>=4,
        % Needs to grab a predefined MPC object
       if isempty(who(mpcname)),
           error('mpc:mpc_loadobj:objnotfound',sprintf(...
               'MPC object "%s" not found',mpcname));
       else
           mpcobj=eval(mpcname);
       end
    else    
        mpcname=mpcobj{1}; % Grab the first one
    end
    return
end

if ~isempty(projname),
    % thisproj must be empty or singleton by design
    %thisproj = find(Projects,'Model',projname,'-depth',0);
    thisproj = find(Projects,'Label',projname,'-depth',0);

    if isempty(thisproj)
        return
    end
else
    % Empty project name
    
    %hWS=Projects;
    mpctasks = Projects.find('-class','mpcnodes.MPCGUI');
    
    if length(mpctasks)==0,
        % No MPC Tasks
        return
    end
    
    errflag=0;
    if length(mpctasks)==1,
        thistask=mpctasks(1);
        thisproj=thistask.up; % This would either be a project 
        % or the workspace (if generated from command line)
    
        if ~isempty(taskname) & ~strcmp(thistask.Label,taskname),
            errflag=1;
        end
        tasklist={thistask.Label};
    else
        tasklist={};
        for i=1:length(mpctasks),
            proji=mpctasks(i).up;
            if ~isempty(proji),
                proji=proji.Label;
                taskname=[mpctasks(i).Label '-' proji];
            else
                taskname=mpctasks(i).Label;
            end
            tasklist{i}=taskname;
        end
        
        if ~isempty(taskname),
            thistask = find(thisproj,'-class', ...
                'mpcnodes.MPCGUI','-depth',1,'Label',taskname);
            if isempty(thistask),
                errflag=1;
            else
                thisproj=thistask.up;
            end
        end
    end    
    if errflag,
        error('mpc:mpc_loadobj:task','MPC task not found');
    end
    
    if isempty(taskname) & length(tasklist)>1,
        % Let the user choose the MPC task
        [choice,ok]=listdlg('PromptString','Choose an MPC design',...
            'SelectionMode','single',...
            'ListString',tasklist,'ListSize',[350 150]);

        thistask=mpctasks(choice);
        thisproj=thistask.up;
    end
end
    
% thistask must be empty or singleton by design
if isempty(taskname) & ~isempty(thisproj),
    thistask = thisproj.getChildren;
end

taskname=thistask.Label;
if ~isempty(thisproj),
    projname=thisproj.Label;
else
    projname=[];
end

if isempty(thistask)
    return
end

if nargin<4,
    % Return list of names of MPC objects
    controllerNodes = find(thistask,'-class','mpcnodes.MPCController');
    if isempty(controllerNodes),
        return
    end
    % Compile a list of MPC objects
    for k=1:length(controllerNodes),
        mpcobj{k} = controllerNodes(k).Label;
    end
    mpcname=mpcobj{1};
else
    % Return MPC object
    controllerNode = find(thistask,'-class','mpcnodes.MPCController','Label',mpcname);
    if isempty(controllerNode),
        errmsg=sprintf('Cannot find MPC controller %s in file %s',...
            mpcname,filename);
        error('mpc:mpc_loadobj:missing',errmsg);
    end
    % Reconstruct MPC object
    mpcobj = controllerNode.getController;
end
